------------------
-- 文件名:	CTimer.lua
-- 版  权:	(C) 
-- 创建人:	李玉平
-- 日  期:	2015-5-15 15:24
-- 版  本:	1.0
-- 描  述:	定时器
-- 应  用:  
-------------------

if g_Timer then
	return 
end

--提供类函数
function classfunc(self, func)
    return function(...)
        func(self, ...)
    end
end

--限制次数的定时器
local CLimitCountTimer = class("CLimitCountTimer")
CLimitCountTimer.__index = CLimitCountTimer

local nTimerID = 0
--fInterval每次时间， nCount总共循环多少次
function CLimitCountTimer:create(nCount, fInterval, func, tbData)
	self.fInterval = fInterval
	self.func = func
	self.nCount = nCount
	self.nCurInterval  = 0
	nTimerID = nTimerID + 1
	self.nID = nTimerID
    self.tbData = tbData
end

--true 则删除该定时器
function CLimitCountTimer:Process(fDeltaTime)
	self.nCurInterval =self.nCurInterval + fDeltaTime
	--cclog(self.nCurInterval.."=============="..self.fInterval)
	if(self.fInterval <= self.nCurInterval)then
		self.nCount  = self.nCount  - 1
		self.nCurInterval  = 0
		if(self.nCount <= 0 )then
			self.func(self.tbData, fDeltaTime, true)
			return true
		else
			return 	self.func(self.tbData, fDeltaTime)
		end
	end
	
	return false
end

--限制时间的定时器
local CLimitTimeTimer = class("CLimitTimeTimer")
CLimitTimeTimer.__index = CLimitTimeTimer
--fInterval每次时间， nCount总共循环多少次
function CLimitTimeTimer:create(nTime, fInterval, func, tbData)
	self.fInterval = fInterval
	self.func = func
	self.nTime = nTime
	self.nCurInterval  = 0
	nTimerID = nTimerID + 1
	self.nID = nTimerID
    self.tbData = tbData
	--self.nCurTime = API_GetCurrentTime()
end

--true 则删除该定时器
function CLimitTimeTimer:Process(fDeltaTime)
	self.nCurInterval =self.nCurInterval + fDeltaTime
	self.nTime = self.nTime - fDeltaTime
	if(self.nTime <= 0)then
		self.func(self.tbData, fDeltaTime, true) --true表示时间到 最后一轮调用
		--cclog(self.nCurTime.."======"..fDeltaTime.."======"..(API_GetCurrentTime()-self.nCurTime))
		return true
	end
	
	if(self.fInterval <= self.nCurInterval)then
		self.nCurInterval  = 0
		return self.func(self.tbData, fDeltaTime)
	end
	
	return false
end

--无限循环定时器
local CLoopTimer = class("CLoopTimer")
CLoopTimer.__index = CLoopTimer
--fInterval每次时间
function CLoopTimer:create(fInterval, func, tbData)
	self.fInterval = fInterval
	self.func = func
	self.nCurInterval  = 0
	nTimerID = nTimerID + 1
	self.nID = nTimerID
    self.tbData = tbData
end

--true 则删除该定时器
function CLoopTimer:Process(fDeltaTime)
	self.nCurInterval =self.nCurInterval + fDeltaTime
	if(self.fInterval <= self.nCurInterval)then
		self.nCurInterval  = 0
		return self.func(self.tbData, fDeltaTime)
	end
	
	return false
end

--创建CTimer类
CTimer = class("CTimer")
CTimer.__index = CTimer

function CTimer:ctor()

	self:clearAllTimer()
end

function CTimer:Update(fDeltaTime)
    for key, classTimers in pairs(self.tbTimer) do
        if classTimers:Process(fDeltaTime) then
            self.tbTimer[key] = nil          
        end
    end
end

function CTimer:destroy()
    self:clearAllTimer()
	self.nID = nil
    self.tbAutoReleasePanelID = {}
end

--定时器 , tbData传入的用户数据，回调原数据返回
function CTimer:pushTimer(fInterval, func, tbData)
	return self:pushLimtCountTimer(1, fInterval, func, tbData)
end

--nTime总共执行多久，fInterval每次间隔的时间 默认是每帧都调用, tbData传入的用户数据，回调原数据返回
function CTimer:pushLimtTimeTimer(nTime,fInterval, func, tbData)
	fInterval = fInterval or 0
	local classTimer = CLimitTimeTimer.new()
	classTimer:create(nTime, fInterval, func, tbData)
	self.tbTimer[classTimer.nID] = classTimer

	return classTimer.nID 
end

--nCount总共执行多少次 fInterval多长时间执行一次, tbData传入的用户数据，回调原数据返回
function CTimer:pushLimtCountTimer(nCount, fInterval, func, tbData)
	local classTimer = CLimitCountTimer.new()
	classTimer:create(nCount, fInterval, func, tbData)
	self.tbTimer[classTimer.nID] = classTimer

	return classTimer.nID 
end

--循环定时器，fInterval每次回调的频率, tbData传入的用户数据，回调原数据返回
function CTimer:pushLoopTimer(fInterval, func, tbData)
	local classTimer = CLoopTimer.new()
	classTimer:create(fInterval, func, tbData)
	self.tbTimer[classTimer.nID] = classTimer

	return classTimer.nID 
end

--清除所有的定时器
function CTimer:clearAllTimer()
	self.tbTimer  = {}
    self.tbAutoReleasePanelID = {}
end

--清除所有的定时器
function CTimer:destroyTimerByID(nID)
    if not nID or nID < 0 then return end

	self.tbTimer[nID] = nil
end

--返回定时器
function CTimer:getTimerByID(nID)
    if not nID or nID < 0 then return end

	return self.tbTimer[nID]
end

--设置窗口关闭时某个定时器自动释放
function CTimer:setTimerAutoRelease(nDestroyTimerID, luaPanel)
    if luaPanel then
	   local panelID = luaPanel
	   self.tbAutoReleasePanelID[panelID] = self.tbAutoReleasePanelID[panelID] or {}
	   table.insert(self.tbAutoReleasePanelID[panelID], nDestroyTimerID)
    end
end

--设置窗口关闭时当前创建的定时器自动释放
function CTimer:setCurTimerAutoRelease(luaPanel)
   self:setTimerAutoRelease(nTimerID, luaPanel) --nTimerID是整个定时器计数器id
end

--设置窗口关闭释放其管理的定时器
function CTimer:onPanelClose(luaPanel)
    if luaPanel then
       local panelID = luaPanel
       local tbAutoReleasePanel = self.tbAutoReleasePanelID[panelID]
       if tbAutoReleasePanel then
          for i=1, #tbAutoReleasePanel do
             self:destroyTimerByID(tbAutoReleasePanel[i])
          end

          self.tbAutoReleasePanelID[panelID] = nil
       end
    end
end

--窗口释放其管理的定时器
function CTimer:releasePanelAllTime(luaPanel)
    self:onPanelClose(luaPanel)
end

--提供给外部使用的定时器接口
--创建定时器, fInterval：时间单位s, luaPanel：面板对象， func：回调函数 tbData:传入的用户数据，回调原数据返回
function CTimer:createTimer(fInterval, luaPanel, func, tbData)
   if luaPanel then
       self:pushTimer(fInterval, classfunc(luaPanel, func), tbData)
       self:setTimerAutoRelease(nTimerID, luaPanel) --nTimerID是整个定时器计数器id
   else
       self:pushTimer(fInterval, func, tbData)
   end
   return nTimerID
end

--nTime:总共执行多久，fInterval:每次间隔的时间,单位s 默认是每帧都调用, tbData:传入的用户数据，回调原数据返回
function CTimer:createLimtTimeTimer(nTime,fInterval, luaPanel, func, tbData)
	fInterval = fInterval or 0
	if luaPanel then
       self:pushLimtTimeTimer(nTime,fInterval, classfunc(luaPanel, func), tbData)
       self:setTimerAutoRelease(nTimerID, luaPanel) --nTimerID是整个定时器计数器id
   else
       self:pushLimtTimeTimer(nTime, fInterval, func, tbData)
   end
   return nTimerID
end

--nCount:总共执行多少次 fInterval:多长时间执行一次,单位s, tbData:传入的用户数据，回调原数据返回
function CTimer:createLimtCountTimer(nCount, fInterval, luaPanel, func, tbData)
   if luaPanel then
       self:pushLimtCountTimer(nCount,fInterval, classfunc(luaPanel, func), tbData)
       self:setTimerAutoRelease(nTimerID, luaPanel) --nTimerID是整个定时器计数器id
   else
       self:pushLimtCountTimer(nCount, fInterval, func, tbData)
   end
   return nTimerID
end

--循环定时器，fInterval:每次回调的频率,单位s, tbData:传入的用户数据，回调原数据返回
function CTimer:createLoopTimer(fInterval, luaPanel, func, tbData)
   if luaPanel then
       self:pushLoopTimer(fInterval, classfunc(luaPanel, func), tbData)
       self:setTimerAutoRelease(nTimerID, luaPanel) --nTimerID是整个定时器计数器id
   else
       self:pushLoopTimer(fInterval, func, tbData)
   end
   return nTimerID
end

g_Timer = CTimer:new()


local function createGloablTimer()
	--[[callbackEntry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(callback, 3, false)
	其中scheduleScriptFunc的三个参数分别是
	callback: 回调函数，Scheduler会传递给回调函数一个参数dt，表示距离上次回调所经过的时间
	delay:每次调用回调函数的时间间隔
	pause: 是否停住，一般设为false就行，否则定时器停住不执行
	]]
	local function callback(fDeltaTime)
	  g_Timer:Update(fDeltaTime)
	  --cc.Director:getInstance():getScheduler():unscheduleScriptEntry(callbackEntry)
	end
	local scheduler = cc.Director:getInstance():getScheduler()
	local schedulerID  = scheduler:scheduleScriptFunc(callback, 0, false)
	
	--cc.Director:getInstance():getScheduler():unscheduleScriptEntry(schedulerID) 
end

--启动定时器
createGloablTimer()