
--[[
math.random 如果不带参数则返回[0,1]范围内的随机数
		如果带一个参数n，将产生1<=X<=n范围内的随机数X
		如果带两个参数a和b，则将产生a<=X<=b范围内的随机数X
		
函数名	 描述	 示例	 结果
pi	 圆周率	 math.pi	 3.1415926535898
abs	 取绝对值	 math.abs(-2012)	 2012
ceil	 向上取整	 math.ceil(9.1)	 10
floor	 向下取整	 math.floor(9.9)	 9
max	 取参数最大值	 math.max(2,4,6,8)	 8
min	 取参数最小值	 math.min(2,4,6,8)	 2
pow	 计算x的y次幂	 math.pow(2,16)	 65536
sqrt	 开平方	 math.sqrt(65536)	 256
mod	 取模	 math.mod(65535,2)	 1
modf	 取整数和小数部分	 math.modf(20.12)	 20   0.12
randomseed	 设随机数种子	 math.randomseed(g_getCursvTime())	  
random	 取随机数	 math.random(5,90)	 5~90
rad	 角度转弧度	 math.rad(180)	 3.1415926535898
deg	 弧度转角度	 math.deg(math.pi)	 180
exp	 e的x次方	 math.exp(4)	 54.598150033144
log	 计算x的自然对数	 math.log(54.598150033144)	 4
log10	 计算10为底，x的对数	 math.log10(1000)	 3
frexp	 将参数拆成x * (2 ^ y)的形式	 math.frexp(160)	 0.625    8
ldexp	 计算x * (2 ^ y)	 math.ldexp(0.625,8)	 160
sin	 正弦	 math.sin(math.rad(30))	 0.5
cos	 余弦	 math.cos(math.rad(60))	 0.5
tan	 正切	 math.tan(math.rad(45))	 1
asin	 反正弦	 math.deg(math.asin(0.5))	 30
acos	 反余弦	 math.deg(math.acos(0.5))	 60
atan	 反正切	 math.deg(math.atan(1))	 45
]]


ccs.COLOR =
{
	GRAY = 	1, 				--灰色
	WHITE = 2, 				--白色
	LIGHT_GREEN = 3, 		--浅绿
	LIME_GREEN = 4,			--深绿
	LIGHT_SKY_BLUE = 5,		--浅蓝
	DARK_SKY_BLUE = 6,		--深蓝
	VIOLET = 7,				--粉色
	BLUE_VIOLET = 8,		--紫色
	FUCHSIA = 9,			--洋红
	DARK_GLOD = 10,			--暗金色
	GLOD = 11,				--金色
	DARK_ORANGE = 12,		--橙色
	ORANG_RED = 13,			--橘红
	RED = 14, 				--纯红
	DEEP_PINK = 15,			--DeepPink
	WHEAT = 16,				--麦色
	PLAIN_ORANGE = 17,		--描述橙色
	PLAIN_YELLOW = 18,		--描述暗黄
	INDIA_RED = 19,			--印度红
	BRIGHT_GREEN = 20,		--亮绿--界面提示
}

local tbtextColor = {
	[ccs.COLOR.GRAY] = cc.c3b(150,150,150), 				--1灰色
	[ccs.COLOR.WHITE] = cc.c3b(255, 255, 255), 				--2白色
	[ccs.COLOR.LIGHT_GREEN] = cc.c3b(144, 238, 144), 			--3浅绿
	[ccs.COLOR.LIME_GREEN] = cc.c3b(35, 220, 55),   			--4深绿
	[ccs.COLOR.LIGHT_SKY_BLUE] = cc.c3b(173, 216, 230), 		--5浅蓝
	[ccs.COLOR.DARK_SKY_BLUE] = cc.c3b(70, 144, 255),   		--6深蓝
	[ccs.COLOR.VIOLET] = cc.c3b(238, 130, 238), 				--7粉色
	[ccs.COLOR.BLUE_VIOLET] = cc.c3b(153, 50, 204),  			--8紫色
	[ccs.COLOR.FUCHSIA] = cc.c3b(255, 0, 255),   				--9洋红
	[ccs.COLOR.DARK_GLOD] = cc.c3b(207, 181, 59),  			--10暗金色
	[ccs.COLOR.GLOD] = cc.c3b(255, 241, 0),   				--11金色
	[ccs.COLOR.DARK_ORANGE] = cc.c3b(255, 140, 0),   			--12橙色
	[ccs.COLOR.ORANG_RED] = cc.c3b(255, 69, 0),    			--13橘红
	[ccs.COLOR.RED] = cc.c3b(255, 0, 0),     					--14纯红
	[ccs.COLOR.DEEP_PINK] = cc.c3b(255, 20, 147),  			--15DeepPink
	[ccs.COLOR.WHEAT] = cc.c3b(245, 222, 179), 				--16麦色
	[ccs.COLOR.PLAIN_ORANGE] = cc.c3b(255, 184, 17),  		--17描述橙色
	[ccs.COLOR.PLAIN_YELLOW] = cc.c3b(181, 174, 156), 		--18描述暗黄
	[ccs.COLOR.INDIA_RED] = cc.c3b(205, 92, 92),   			--19印度红
	[ccs.COLOR.BRIGHT_GREEN] = cc.c3b(50, 255, 50),   		--20亮绿--界面提示
	
	[21] = cc.c3b(144, 238, 144),-- 21亮绿
	[22] = cc.c3b(255, 165, 0), -- 22棕黄 
	[23] = cc.c3b(0, 0, 0),-- 23黑色
	[24] = cc.c3b(165, 42, 42),-- 24血红
	[25] = cc.c3b(255, 255,0),-- 25 纯黄
	[26] = cc.c3b(103, 180,255),-- 26 蓝
	[27] = cc.c3b(191,191,191), 				--27浅灰色
	[28] = cc.c3b(60,60,60), 				--28深灰色
	[29] = cc.c3b(255,255,132), 				--29 亮黄..
	[30] = cc.c3b(255,213,132), 				--29 亮棕黄.
	[31] = cc.c3b(102,51,0), 				--30 中
	[32] = cc.c3b(102,51,255), 				--30 紫
	[33] = cc.c3b(0,51,255), 				--30 蓝
	[34] = cc.c3b(51,51,51), 				--30 浅灰
} 

function cc.g_getColor(nColorType)
	return tbtextColor[nColorType]
end


--[[
	设置字体颜色
	@param txt 要设置的对象
	@param index 1-20分别为不同的颜色，如果为空默认为白色
	
]]
function cc.g_setTextColor(txt,index,isFGUI)
	if not txt then
        cclog("txt为空了 （g_setTextColor）=="..ind232ex) 
        return 
    end 
	if  isFGUI  then
		txt = txt:displayObject()
	end 
	
	if not index then cclog("index为空了 （g_setTextColor）==")  index = ccs.COLOR.WHITE end
	txt:setColor(tbtextColor[index])
end


--textfield输入内容区分字母或者数字
function cc.isNumberOrCharacter(mystring) 
    for i = 1,#mystring do 
        local account = string.sub(mystring,i,i) 
        local temp = string.byte(account) 
                 
        if 48 <= temp and temp <= 57 then                                
            --cclog("这是数字") 
        elseif 65<= temp and temp <= 90 then 
            --cclog("这是大写字母") 
            return false 
        elseif 97 <= temp and temp <= 122 then 
           	--cclog("这是小写写字母") 
           	return false 
		elseif 32 == temp then 
           	--cclog("这是空格") 
           	return false 
        else 
            --cclog("输入包含非数字、字母的字符，不符合规则，请重新输入") 
            return false 
        end 
    end 
	return true	
end 
--是否是中文
function cc.isChinese(mystring) 
    for i = 1,#mystring do 
        local account = string.sub(mystring,i,i) 
        local temp = string.byte(account) 
		
        if (temp >= 0 and temp <= 127) then                                
        else 
            return true 
        end 
    end 
	return false	
end
--textfield框计算字数  英文算1个字，中文2个字
function cc.stringNum(mystring,maxNumber) 
	local num = 0
	local list = {}
    local nlen = tonumber( string.len(mystring))
    local i = 1 
	local maxString = ""
	while i <=  nlen do
		local c = string.byte(mystring, i)
        local shift = 1
        if c > 0 and c <= 127 then
            shift = 1
			num = num + 1
        else
			shift = 3
			num = num + 2
        end
      --  local char = string.sub(mystring, i, i+shift-1)
		if maxNumber and num >= maxNumber then
			maxString =  string.sub(mystring, 1, i+shift-1)
			return num,maxString
		end
		i = i + shift
    end 
	return num,maxString
end

--截取字段
function cc.stringSub(mystring,startIndex,endIndex) 
	if not mystring or mystring == "" then
		return ""
	end 
	local num = 0
	local status = true
    local nlen = tonumber( string.len(mystring))
    local i = 1 
	local nString 
	startIndex = startIndex or 1
	endIndex = endIndex or 1
	if startIndex >  endIndex then
		ccolg("=Eorro==startIndex >  endIndex===")
		return
	end
	
	while i <=  nlen do
		local c = string.byte(mystring, i)
        local shift = 1
        if c > 0 and c <= 127 then
            shift = 1
			num = num + 1
        else
			shift = 3
			num = num + 2
        end
      --  local char = string.sub(mystring, i, i+shift-1)
		if startIndex and num == startIndex then
			startIndex = i
		end
		if endIndex and num >= endIndex then
			endIndex = i
			nString =  string.sub(mystring, startIndex,endIndex+shift-1)
			return nString
		end
		if  i + shift >  nlen  then
			endIndex = i
			nString =  string.sub(mystring, startIndex,endIndex+shift-1)
			return nString
		end
		i = i + shift
    end 
	return nString
end

-----------------textfield输入内容区分字母或者数字   ---
function cc.Textisblank(mystring) 
    for i = 1,#mystring do 
        local account = string.sub(mystring,i,i) 
        local temp = string.byte(account) 
                 
        if 32 == temp then 
           	--cclog("这是空格") 
        else 
            cclog("输入包含非数字、字母的字符，不符合规则，请重新输入") 
            return false 
        end 
    end 
	return true	
end 

---判断位数--------
function cc.PrintDigits(num)
	local digits = math.ceil(math.log(num)/math.log(10))
	return digits 
end

--插入换行符或其他
function cc.g_string_insert(mystring,insertStr,mlen)
	local b =  tonumber( string.len(insertStr))
    local newstr = ""; 
	local num = 0
    local nlen = tonumber( string.len(mystring))
    local i = 1 
	while i <=  nlen do
		local c = string.byte(mystring, i)
        local shift = 1
        if c > 0 and c <= 127 then
            shift = 1
			num = num + 1
        else
			shift = 3
			num = num + 2
        end
        local substr = string.sub(mystring, i, i+shift-1)
        i = i + shift
		if num < mlen*2 then
			newstr = newstr .. substr
		else
			local substr1 = string.sub(mystring, i, i)
			num = 0
			if substr1 == "\n" then
				newstr = newstr .. substr..substr1
				i = i + 1
			else
				newstr = newstr .. substr..insertStr
			end
		end
    end 
    return newstr; 
end 
--[[
--控制label长度换行
function cc.g_stringSize_insert(mystring,insertStr,fontSize,mSizeLen) 
	local b =  tonumber( string.len(insertStr))
    local newstr = ""; 
	local size = 0
    local nlen = tonumber( string.len(mystring))
    local i = 1 
	if not g_nlabel then
		g_nlabel = Label:create()
		g_nlabel:retain()
	end
	g_nlabel:setFontSize(fontSize)
	while i <=  nlen do
		local c = string.byte(mystring, i)
        local shift = 1
        if c > 0 and c <= 127 then
            shift = 1
        else
			shift = 3
        end
        local substr = string.sub(mystring, i, i+shift-1)
        i = i + shift
		local text = newstr .. substr.."\n"
		g_nlabel:setText(text)
		local size = g_nlabel:getSize()
		if size.width <= mSizeLen then
			newstr = newstr .. substr
		else
			local substr1 = string.sub(mystring, i, i)

			if substr1 == "\n" then
				newstr = newstr ..substr1..substr
				i = i + 1
			else
				newstr = newstr .."\n".. substr
			end
		end
    end 
    return newstr; 
end 




local tbSoundRelease = {}
function g_playSoundEffect(path)	--后面为了区分安卓和苹果的声音文件只传入文件名即可
	local IsMuteSoundEffet = CCUserDefault:sharedUserDefault():getBoolForKey("IsMuteSoundEffet", false)
	if g_Cfg.Platform == kTargetWindows then
		if IsMuteSoundEffet then
			cclog("IsMuteSoundEffet In Windows")
			return nil --静音模式不播放音效
		else
			tbSoundRelease[path] = true
			return SimpleAudioEngine:sharedEngine():playEffect(path)
		end
	else
		if IsMuteSoundEffet then
			return nil --静音模式不播放音效
		else
			tbSoundRelease[path] = true
			return SimpleAudioEngine:sharedEngine():playEffect(path)
		end
	end
end

function g_playSoundEffectBattle(path)
    local IsMuteBattleSound = CCUserDefault:sharedUserDefault():getBoolForKey("IsMuteBattleSound", false)
	if g_Cfg.Platform == kTargetWindows then
		if IsMuteBattleSound then
			return nil --战斗静音不播放音效
		else
			tbSoundRelease[path] = true
			return SimpleAudioEngine:sharedEngine():playEffect(path)
		end
	else
		if IsMuteBattleSound then
			return nil --战斗静音不播放音效
		else
			tbSoundRelease[path] = true
			return SimpleAudioEngine:sharedEngine():playEffect(path)
		end
	end
end


function g_unloadEffect()
	for key, value in pairs(tbSoundRelease) do
		SimpleAudioEngine:sharedEngine():unloadEffect(key)
	end
	
	tbSoundRelease = {}
end

function g_playSoundMusic(path, bLoop)
    local IsMuteSoundMusic = CCUserDefault:sharedUserDefault():getBoolForKey("IsMuteSoundMusic", false)
    if g_Cfg.Platform == kTargetWindows then
		if path then--表示重置音乐
            SimpleAudioEngine:sharedEngine():playBackgroundMusic(path, bLoop)
        end
        if IsMuteSoundMusic then
	   	   SimpleAudioEngine:sharedEngine():pauseBackgroundMusic()
        else
           SimpleAudioEngine:sharedEngine():resumeBackgroundMusic()
        end
    else
        if path then--表示重置音乐
            SimpleAudioEngine:sharedEngine():playBackgroundMusic(path, bLoop)
        end
        if IsMuteSoundMusic then
	   	   SimpleAudioEngine:sharedEngine():setBackgroundMusicVolume(0)
        else
           SimpleAudioEngine:sharedEngine():setBackgroundMusicVolume(1)
        end
    end
end

function g_playSoundMusicBattle(path, bLoop)
    local IsMuteBattleMusic = CCUserDefault:sharedUserDefault():getBoolForKey("IsMuteBattleMusic", false)
    if g_Cfg.Platform == kTargetWindows then
		if path then--表示重置音乐
            SimpleAudioEngine:sharedEngine():playBackgroundMusic(path, bLoop)
        end
        if IsMuteBattleMusic then
	   	   SimpleAudioEngine:sharedEngine():pauseBackgroundMusic()
        else
           SimpleAudioEngine:sharedEngine():resumeBackgroundMusic()
        end
    else
        if path then--表示重置音乐
            SimpleAudioEngine:sharedEngine():playBackgroundMusic(path, bLoop)
        end
        if IsMuteBattleMusic then
	   	   SimpleAudioEngine:sharedEngine():setBackgroundMusicVolume(0)
        else
           SimpleAudioEngine:sharedEngine():setBackgroundMusicVolume(1)
        end
    end
end
]]

--字符串换行加%s替换
function cc.g_initMsgContent(text,mLen,tb_str,tb_number) 
	local index = 1  
	local newStr = ""
	local subStr = ""
	local strIndex = 1
	local numIndex = 1
	local is_end = true
	local index_max =  string.len(text)
	while (is_end) do  
		local temp = string.byte(text,index) 
        if (temp >= 0 and temp <= 127) then 
			subStr =string.sub(text,index,index) 
  			if subStr =="%" then
				local nextStr =string.sub(text,index+1,index+1) 
				if nextStr == "s" then
					subStr = tb_str[strIndex]
					strIndex = strIndex + 1
					--cclog("==='s'==="..(index+1))
				elseif nextStr == "d" then
					subStr = tb_number[numIndex]
					numIndex = numIndex + 1
					--cclog("==='s'==="..(index+1))
				end
				
				index = index + 2 
			else
			index = index + 1 
			end
        else 		
			subStr =string.sub(text,index,index + 2) 
			index = index + 3
        end  
		newStr = newStr..subStr
        if (index>=index_max) then  
            is_end=false  
        end
    end 	
	str_chat = g_string_insert(newStr,"\n",mLen)
	return str_chat
end

--字符串分割  里面有 x 字符 分割前后 富文本
-- 内容 宽度 替换数字组 富文本  UIText 间隔
--noRichText 不需要富文本
function cc.g_richText_inalienContent(text,widthNum,tb_number,uText,offsetX,offsetY,parent) 
	local tbText = {[1]={}}
	local offsetX = offsetX or 0
	local offsetY = offsetY or 0
	local num = 1
	local indexH = 1
	local index = 1  
	local newStr = ""
	local subStr = ""
	local strIndex = 1
	local numIndex = 1
	local is_end = true
	local index_max =  string.len(text)
	local _richText,UIText
	local tbNumberText ={}
	if not richText then
		_richText = {}
		_richText = {[1] = {} }
		
	else
		_richText = richText
	end 
	
	UIText = uText or _richText[1][1]
	local fontSize = fontSize or UIText:getFontSize()
	local parent = parent or UIText:getParent()
	local posX,PosY = UIText:getPosition()
	--_richText[1][1] = UIText
	--g_setAnchorPoint(UIText,0,1)
	local function setRichText(indexH,indexJ,text,textName)
		if not _richText[indexH] then
			_richText[indexH] = {}
		end 
		
		local re1 = _richText[indexH][indexJ]
		if not re1 then
			
			re1 = ccui.Text:create(text, G_path.fontPath, fontSize)
			_richText[indexH][indexJ] = re1
			re1:setTag(indexH*100+indexJ)
			parent:addChild(re1)
		else
			--cclog(indexH,indexJ,"=========1111===========")
			re1:setString(text)
			re1:setFontSize(fontSize)
		end 
		if textName then
			tbNumberText[#tbNumberText+1] = re1
		end 
	end 
	
	while (is_end) do  
		local temp = string.byte(text,index) 
        if (temp >= 0 and temp <= 127) then 
			subStr =string.sub(text,index,index) 
  			if subStr =="x" then
				local curSubStr =string.sub(text,strIndex,index-1) 
				local indexK = #tbText[indexH]+1
				tbText[indexH][indexK] = curSubStr
				setRichText(indexH,indexK,curSubStr)
				local number = tb_number[numIndex]
				--cclog(number,numIndex,tb_number,indexH,indexK,curSubStr,strIndex,index,"====g_richText_inalienContent======number========")
				if not number then
					number= tb_number[1]
				end 
				local digits = math.ceil(math.log(number)/math.log(10))
				if num + digits > widthNum then
					--cclog(number,numIndex,tb_number,digits,num + digits ,widthNum"==========number====11====")
					indexH = indexH + 1
					tbText[indexH] = {}
					tbText[indexH][1] = number
					setRichText(indexH,1,number,"x"..numIndex)
					num = digits
				else
					--cclog(number,numIndex,tb_number,digits,"==========number==222======")
					local indexK = #tbText[indexH]+1
					tbText[indexH][indexK] = number
					setRichText(indexH,indexK,number,"x"..numIndex)
					num = num + digits
				end 
				numIndex = numIndex + 1
				index = index + 1
				strIndex = index
			elseif subStr =="n" then
				local curSubStr =string.sub(text,strIndex,index-1) 
				local indexK = #tbText[indexH]+1
				tbText[indexH][indexK] = curSubStr
				setRichText(indexH,indexK,curSubStr)
				index = index + 1
				strIndex = index
				indexH = indexH + 1
				tbText[indexH] = {}
				--local text = "我我我我我我"
				--tbText[indexH][1] = text
				--setRichText(indexH,1,text)
				num = 0
			else
				if num + 1 > widthNum then
					local curSubStr =string.sub(text,strIndex,index-1) 
					indexH = indexH + 1
					tbText[indexH] = {}
					tbText[indexH][1] = number
					setRichText(indexH,1,number,"x"..numIndex)
					strIndex = index
					num = 1
				else
					num = num + 1
				end
				index = index + 1 
			end
        else 		
			if num + 2 > widthNum then
				local curSubStr =string.sub(text,strIndex,index-1) 
				local indexK = #tbText[indexH]+1
				tbText[indexH][indexK] = curSubStr
				setRichText(indexH,indexK,curSubStr)
				strIndex = index
				indexH = indexH + 1
				tbText[indexH] ={} 
				num = 0
			else
				num = num + 2
			end 
			index = index + 3
        end  
        if (index>=index_max) then  
			local curSubStr =string.sub(text,strIndex,index_max) 
			local indexK = #tbText[indexH]+1
			tbText[indexH][indexK] = curSubStr
			setRichText(indexH,indexK,curSubStr)
            is_end=false  
        end
    end 	
		
	local nextText = UIText
	for i,v in ipairs(_richText)do
		--cclog(#_richText,#v,"=======11111==i,j,n,======")
		--if i>1 then
			v[1]:setAnchorPoint(0,1)
			local posX,posY = nextText:getPosition()
			local nSize = nextText:getContentSize()
			local nPosX = posX
			local nPosY
			if  i == 1 then
				nPosY = posY 
			else
				nPosY = posY - nSize.height - offsetY
			end 
			v[1]:setPosition(nPosX,nPosY)
			nextText = v[1]
		--end
		--cclog(_richText[1],v[1],"=========i,j,n,======")
		g_setTbTexts(v,offsetX)
	end 
	--cclog(tbNumberText,"=======")
	return _richText,tbNumberText
end

--字符串分割 替换  里面有 x 字符 
-- 内容 替换字符组  替换数字组  间隔
function cc.g_inalienContentByTbNumber(text,tb_Str,tb_number,color)  
	local tbText = {[1]={}}
	local index = 1  
	local newStr = ""
	local subStr = ""
	local strIndex = 1
	local is_end = true
	local index_max =  string.len(text)
	
	while (is_end) do  
		local temp = string.byte(text,index) 

        if (temp >= 0 and temp <= 127) then 
			subStr =string.sub(text,index,index) 
			
			for i,cStr in ipairs(tb_Str)do	
				
				if subStr == cStr then
					local curSubStr =string.sub(text,strIndex,index-1) 
					
					local number = tb_number[i]
					
					if not number then
						number= tb_number[1]
					end 
					strIndex = index + 1
					if color then
						local colorStr1 =string.format("[color=%s]", color)
						local colorStr2 = "[/color]"
						newStr = newStr..curSubStr..colorStr1..number..colorStr2
					else
						newStr = newStr..curSubStr..number
					end 
				else
					
				end
				
			end 
  			index = index + 1
        else 	

			index = index + 3	
        end  
        if (index>index_max) then  
			is_end=false  
        end
    end 	
	if strIndex <= index_max then
		local subStr =string.sub(text,strIndex,index_max)
		newStr = newStr..subStr
	end 
	return newStr
end

---获取当月天数
function cc.getDatesNum(year,month)
	local Num
	local temp = os.date("*t")
	local year= year or temp.year
	local month= month or temp.month
	local day=1
	local hour=0
	local nextMonth 
	if month ~= 12 then
		nextMonth = month + 1
		local nTime = os.time{year = year, month = nextMonth, day=1, hour=0}
		nTime = nTime - 60*60
		temp = os.date("*t", nTime)
		Num = temp.day
	else
		Num = 31
	end
	return Num
 end

 --获取距离xxx的时间
function cc.getStrTime(nTime,curTime,count)
	local curTime = curTime or g_getCursvTime()
	
	local nTime = nTime or 0
	nTime = math.abs(nTime)
	nTime = math.floor(nTime)
	local text = math.floor((curTime - nTime) / 60)
	if text < 1 then
		text = "刚刚"
		return text
	elseif text >=1 and text < 60 then
		text = text.."分钟前"
		return text
	end
	text = math.floor((curTime - nTime) / (60*60))
	if text >=1 and text < 24 then
		
		if count then
			local text2 = (curTime - nTime) % 60
			text2 = math.floor(text2 / 60)
			text = text.."小时"..text2.."分钟前"
		else
			text = text.."小时前"
		end 
		return text
	end
	text = math.floor((curTime - nTime) / (24*60*60))
	if text >=1 and text < 7 then
		text = text.."天前"
		return text
	end
	text = math.floor((curTime - nTime) / (7*24*60*60))
	if text >=1 and text <5 then
		text = text.."周前"
		return text
	end
	text = math.floor((curTime - nTime) / (30*24*60*60))
	if text >=1 and text <12 then
		text = text.."月前"
		return text
	end
	text = math.floor((curTime - nTime) / (12*30*24*60*60))
	if text >=1 then
		text = text.."年前"
		return text
	end
	return text
end



--把秒数换成时间
function cc.getStrTimeByS4(nTime,bShow)
	local nTime = nTime or g_getCursvTime()
	nTime = math.abs(nTime)
	nTime = math.floor(nTime)
	local s = nTime % 60
	local m = math.floor( nTime / 60 )%60
	local h = math.floor( nTime / 3600)%24
	local d = math.floor( nTime / (3600*24))
	local text = ""
	if d > 0 then
		text= d.."天"..h.."小时"
		if h == 0 and bShow then
			text = d.."天"
		end 
		return text
	end 
	if h > 0 then
		text= h.."小时"..m.."分"
		if m == 0 and bShow then
			text = h.."小时"
		end 
		return text
	end 
	if m > 0 then
		text= m.."分"..s.."秒"
		if s == 0 and bShow then
			text = m.."分"
		end 
		return text
	end 
	text= s.."秒"
	return text
end

--把秒数换成时间
function cc.getStrTimeByS(nTime,maxCount)
	local nTime = nTime or g_getCursvTime()
	nTime = math.abs(nTime)
	nTime = math.floor(nTime)
	local s = nTime % 60
	local m = math.floor( nTime / 60 )%60
	local h = math.floor( nTime / 3600)%24
	local d = math.floor( nTime / (3600*24))
	local text = ""
	if d > 0 then
		
		if not maxCount then
			text= d.."天"..h.."小时"..m.."分"..s.."秒"
		elseif maxCount == 3 then
			text= d.."天"..h.."小时"..m.."分"
		elseif maxCount == 2 then
			text= d.."天"..h.."小时"
		elseif maxCount == 1 then
			text= d.."天"
		else
			text= d.."天"..h.."小时"..m.."分"..s.."秒"
		end 
		return text
	end 
	if h > 0 then
		if not maxCount then
			text= h.."小时"..m.."分"..s.."秒"
		elseif maxCount == 2 then
			text= h.."小时"..m.."分"
		elseif maxCount == 1 then	
			text= h.."小时"
		else
			text= h.."小时"..m.."分"..s.."秒"
		end 
		return text
	end 
	if m > 0 then
		if  maxCount == 1 then
			text= m.."分钟"
		else
			text= m.."分"..s.."秒"
		end 
		
		return text
	end 
	text= s.."秒"
	return text
end


--把秒数换成时间2
function cc.getStrTimeByS2(nTime)
	local nTime = nTime or g_getCursvTime()
	nTime = math.abs(nTime)
	nTime = math.floor(nTime)
	local s = nTime % 60
	local m = math.floor( nTime / 60 )%60
	local h = math.floor( nTime / 3600)%24
	local d = math.floor( nTime / (3600*24))
	local text = ""
	if d > 0 then
		text= d.."d"..h.."h"..m.."m"..s.."s"
		return text
	end 
	if h > 0 then
		text= h.."h"..m.."m"..s.."s"
		return text
	end 
	if m > 0 then
		text= m.."m"..s.."s"
		return text
	end 
	text= s.."s"
	return text
end

--把秒数换成时间3
function cc.getStrTimeByS3(nTime,nType,maxCount)
	if not nTime  then	
		return
	end 
	local nTime = nTime or g_getCursvTime()
	nTime = math.abs(nTime)
	nTime = math.floor(nTime)
	local s = nTime % 60
	local m = math.floor( nTime / 60 )%60
	local h = math.floor( nTime / 3600)%24
	local d = math.floor( nTime / (3600*24))
	local text = ""
	
	local function addZero(value)
		if value < 10 then
			value = "0"..value
		end
		return value 
	end 
	if d > 0 then
		s = addZero(s)
		m = addZero(m)
		h = addZero(h)
		if not maxCount then
			text= d..":"..h..":"..m..":"..s
		elseif  maxCount == 3 then
			text= d..":"..h..":"..m
		elseif maxCount == 2 then
			text= d..":"..h	
		else
			text= d..":"..h..":"..m..":"..s
		end 
		return text
	end 
	if h > 0 then
		s = addZero(s)
		m = addZero(m)
		text= h..":"..m..":"..s
		if not maxCount then
			text= h..":"..m..":"..s
		elseif maxCount == 2 then
			text= h..":"..m
		else
			text= h..":"..m..":"..s
		end 
		return text
	end 
	if m > 0 then
		s = addZero(s)
		text= m..":"..s
		return text
	end 
	if not nType then
		local s = nTime % 60
		text = s
	else
		s = addZero(s)
		text= "00:"..s
	end 

	return text
end

 --裁剪
function g_setClippingNode(ccSpriteStencil,Stencil,parent,pos)
	local pos = pos or cc.p(0,0)
	local clippingNode = cc.ClippingNode:create()
--	local clippingNode = cc.ClippingRectangleNode:create(cc.rect(0,0,100,100));
	parent:addChild(clippingNode,1)

	clippingNode:setStencil(Stencil)
	clippingNode:setAlphaThreshold(0)
	clippingNode:setPosition(pos)
	--clippingNode:setInverted(false)
	clippingNode:addChild(ccSpriteStencil,10)
	
	return clippingNode
end
--渲染效果
function g_setBlendFuncDstColor(Sprite,blendFuncType,noCCUI)
	if noCCUI then
		Sprite = Sprite:getVirtualRenderer()	
	end 
	local blendFuncMinusDstColor = cc.blendFunc()
	blendFuncMinusDstColor.src = gl.ONE_MINUS_DST_COLOR
	blendFuncMinusDstColor.dst = gl.ONE
	local blendFuncDstColor = cc.blendFunc()
	blendFuncDstColor.src = gl.DST_COLOR
	blendFuncDstColor.dst = gl.ONE
	local blendFuncSrcAlpha = cc.blendFunc()
	blendFuncSrcAlpha.src = gl.SRC_ALPHA
	blendFuncSrcAlpha.dst = gl.ONE
	--local blendFuncType = blendFuncType or 2
	if blendFuncType == 1 then
		Sprite:setBlendFunc(blendFuncMinusDstColor)
	elseif   blendFuncType == 2 then
		Sprite:setBlendFunc(blendFuncDstColor)
	else
		Sprite:setBlendFunc(blendFuncSrcAlpha)
	end
end
--深度拷贝
function deepcopy(tDest, tSrc) 
	for key,value in pairs(tSrc) do 
		if type(value)=='table' and value["spuer"]==nil then 
			tDest[key] = {} 
			deepcopy(tDest[key],value) 
		else 
			tDest[key]=value
		end 
	end 
	--cclog(tDest,"===deepcopy====",tSrc)
	return tDest
end
--克隆拷贝
function g_clone( object )
    local lookup_table = {}
    local function copyObj( object )
        if type( object ) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        
        local new_table = {}
        lookup_table[object] = new_table
        for key, value in pairs( object ) do
            new_table[copyObj( key )] = copyObj( value )
        end
        return setmetatable( new_table, getmetatable( object ) )
    end
    return copyObj( object )
end


--获取倒计时  ntye = 1 输出为00  2:输出为 00:00  3为00:00:00
function getCoolTimeStr1(starTime,coolTime,maxCount)
	if not starTime then
		return 
	end
	local curTime  = g_getCursvTime()
	local nTime1 = curTime - starTime
	local nTime = coolTime - nTime1
	--cclog(coolTime,"===getCoolTimeStr1=",nTime,nTime1)
	if nTime < 0 then
		nTime = 0
	end 
	
	local  coolTime_Str =  cc.getStrTimeByS(nTime,maxCount)
	return coolTime_Str,nTime
end

function getCoolTimeStr(coolTime,ntye)
	local coolTime_Str = 0
	local coolTime = math.ceil(coolTime)
	local s = coolTime%60
	local m = math.floor(coolTime/60)%60
	local h = math.floor(coolTime/3600)%60

	local ntye = ntye or 2
	if s < 10 then
		s = "0"..s
	end 
	if m < 10 then
		m = "0"..m
	end 
	coolTime_Str = m..":"..s
	
	if ntye == 1 then
		
	elseif ntye == 2 then
		if h > 0 then
			coolTime_Str = h..":"..m
		end 
	elseif ntye == 3 then
		if h > 0 then
			coolTime_Str =  h..":"..m..":"..s
		end 
		
	end 
	return  coolTime_Str,coolTime
end
-- 加载图片
function g_loadTexture(image,icon,noPlist)
	image:ignoreContentAdaptWithSize(true) 	
	if not noPlist then
		image:loadTexture(icon,g_PlistType1)
	else
		image:loadTexture(icon,g_PlistType)
	end 
	
end
-- 设置文本  自动换行
function g_setString(text,nString,width)	
	text:setString("")	
	--text:setContentSize(cc.size(0,0)) 
	text:ignoreContentAdaptWithSize(true) 
	
	text:setTextAreaSize(cc.size(0,0))
	text:setString(nString) 
	local textSize = text:getContentSize() 
	local nWidth = width
	if textSize.width < width then
		nWidth = textSize.width
		return
	end 

	text:setTextAreaSize(cc.size(nWidth,0))
	text:setString(nString) 
	--text:ignoreContentAdaptWithSize(false) 
end
-- 设置文本  控件间的间隔
-- interval >0 在text右边，<=0在text左边
-- nType 是否居中
function setIntervalByText(sprit,text,interval,offsetX,nType,curPosX)	
	local text_anchorPoint = text:getAnchorPoint()
	--local sprit_anchorPoint = sprit:getAnchorPoint()
	--sprit:setAnchorPoint(cc.p(0.5,0.5))
	--text:setAnchorPoint(cc.p(0.5,0.5))
	--g_setAnchorPoint(sprite,0.5,0.5)
	
	local tPosX,tPosY = text:getPosition()
	local sPosX,sPosY = text:getPosition()
	if curPosX then
		tPosX = curPosX
		sPosX = curPosX
	end 
	local size = text:getContentSize()
	local offsetX = offsetX or 0
	local scaleX1= text:getScaleX()
	local interval = interval or 0
	local textWidth = 0
 	
	if 	text_anchorPoint.x == 1 then
		if interval < 0 then
			textWidth = size.width
		else
			textWidth = 0
		end 
		
	elseif text_anchorPoint.x == 0  then
		if interval < 0 then
			textWidth = 0
		else
			textWidth = size.width
		end 
	elseif 	text_anchorPoint.x == 0.5 then
		textWidth = size.width/2
	end
	
	if interval and interval < 0 then
		sPosX = tPosX - textWidth + offsetX
	else
		sPosX = tPosX + textWidth + offsetX
	end 
	sprit:setPosition(sPosX,sPosY)
	--sprit:setAnchorPoint(sprit_anchorPoint)
	--text:setAnchorPoint(text_anchorPoint)
	
	
	if nType then
		local scaleX2= sprit:getScaleX()
		local size2 = sprit:getContentSize()
		local width = size2.width*scaleX2 + size.width + offsetX
		width = width/2
		local count  = size.width - width
		if interval < 0 then
			count = 0-count
		end 
		sprit:setPosition(sPosX+count,sPosY)
		text:setPosition(curPosX+count,tPosY)
		
		
	end
	
	return count,size
end 

--获取平台信息 
-- 参数1：平台id ：  
-- 参数2：回调 
function setPlatformCallBack(platformStr,callBack)	
	if not g_Cfg.Platform then
	
		local Platform = cc.Application:getInstance():getTargetPlatform()
		g_Cfg.Platform = Platform
	end 

	cclog(kTargetWindows,kTargetIphone,kTargetAndroid,kTargetIpad,"===setPlatformCallBack =======",platformStr, g_Cfg.Platform)
	if g_Cfg.Platform == kTargetWindows  then --wind
		if callBack and platformStr == "windows" then
			callBack()
		end 
	elseif  g_Cfg.Platform ==  kTargetAndroid  then
		if callBack and platformStr == "android"  then
			callBack()
		end 
	else
		if iosFunc then
			if callBack and platformStr == "ios"  then
				callBack()
			end 		
		else
			cclog("===iosFunc is nil =======")
		end 
	end
end



function setPlatformCallBack_NoPlatform(notPlatformStr,callBack)	 
	--cclog(g_Cfg.Platform,kTargetWindows,"===notPlatformStr =======",notPlatformStr)
	if g_Cfg.Platform == kTargetWindows  then --wind
		if callBack and notPlatformStr ~= "windows" then
			callBack()
		end 
	elseif  g_Cfg.Platform ==  kTargetAndroid   then
		if callBack and notPlatformStr ~= "android"  then
			callBack()
		end 
	elseif  g_Cfg.Platform ==  kTargetIpad  then
		if callBack and notPlatformStr ~= "ios"  then
			callBack()
		end 
	elseif  g_Cfg.Platform ==  kTargetIphone  then
		if callBack and notPlatformStr ~= "ios"  then
			callBack()
		end 
	end
end


--[[
enticationStatus = 0
function enticationStatus(szBuf)
    local tbmsg = assert(loadstring(szBuf))()
    cclog("====登录===="..tbmsg.isSuccess)
    if tbmsg.isSuccess == 1 then
        cclog("====登录成功===="..tbmsg.isSuccess)
        enticationStatus = 1
    else
        enticationStatus = 0
        cclog("====登录失败===="..tbmsg.isSuccess)
    end
end]]
--分数上传
function reportScoreCallback(szBuf)
    local tbmsg = assert(loadstring(szBuf))()
    --cclog("====分数上传成功===="..tbmsg.isSuccess)
    if tbmsg.isSuccess == 1 then
        --cclog("====分数上传成功===="..tbmsg.isSuccess)
    end
end
--设置锚点 忽略锚点对位置的影响 
function g_setAnchorPoint(sprite,anPosX,anPosY)
	
	--[[
	local curAnPos = sprite:getAnchorPoint()
	local curPosX,curPosY = sprite:getPosition()
	local size = sprite:getContentSize()
	local swapAnPosX = anPosX-curAnPos.x
	local swapAnPosY = anPosY-curAnPos.y
	local toPosX = curPosX + size.width*swapAnPosX
	local toPosY = curPosY + size.height*swapAnPosY
	sprite:setPosition(toPosX,toPosY)]]
	sprite:ignoreAnchorPointForPosition(false)
	sprite:setAnchorPoint(cc.p(anPosX,anPosY))
end 

--链接 几个文本 -- 文本， 间隔,向右或左
--喵点 0.5,0.5
function g_setTbTexts(tbText,space,nType)
	local nType = nType or 1
	local space = space or 0
	local anPosX = 0 
	if nType == 1 then --向右
		anPosX = 0
	elseif nType == 0 then
		nType = -1    --向左
		anPosX = 1
	elseif nType == -0.5 then
		nType = -0.5    --向左 锚点0.5
		anPosX = 0.5
	elseif nType == 0.5 then
		nType = 0.5    --向右 锚点0.5
		anPosX = 0.5
	end 
	local startText = tbText[1]
	g_setAnchorPoint(startText,anPosX,0.5)
	local countWidth = 0
	for i=2,#tbText do
		g_setAnchorPoint(tbText[i],anPosX,0.5)
		startText = tbText[i-1]
		local nSize = startText:getContentSize()
		local posX,posY = startText:getPosition()
		local curPosX = posX + nType*nSize.width + nType*space
		tbText[i]:setPosition(curPosX,posY)
		countWidth = countWidth + nType*nSize.width + nType*space
	end 
	 return countWidth
end 

--判断有小数 还是整数, 如果小数 保留几位
function g_isInt(number,nums,bStr)
	if not number then return end 
	number = tonumber(number)
	local value = g_roundNumber(number)
	if nums == 0 then 
		return value
	end 
	if value ~= number then
		if nums then
			local value = tonumber(string.format("%0."..nums.."f",number))
			if bStr then
				local value1 = value*math.pow(10,nums)
				if value1 == number then
					value = value.."."
					for i=1,nums do
						value = value.."0"
					end
				end 
			end
			return value
		end
		return number,false
	else
		return number,true
	end 
end 

--判断有小数  四舍五入
function g_roundNumber(number)
	if number == math.floor(number) then
		return number
	end 
	if number < 0  then
		number = math.floor(number - 0.5)
	else
		number = math.floor(number + 0.5)
	end 
	return number
end 

--getWorildPos  ---世界坐标   world
function getWorildPos(item,offsetX,offsetY)
	local Parent =  item:getParent() 
	local curPosX,curPosY = item:getPosition()
	if not Parent then
		return
	end 
	local toPos = Parent:convertToWorldSpace(cc.p(curPosX,curPosY))
	local offsetX = offsetX or 0
	local offsetY = offsetY or 0
	toPos = cc.p(toPos.x+offsetX,toPos.y+offsetY)
	return toPos
end

--增加百分比
function g_addPercent(baseValue,addV)
	local dpsValue
	local addV = addV or 0
	dpsValue = baseValue*(1+addV/100)
	return dpsValue
end 

--wangluo时间
function g_getNetTime()
	g_NetTime = nil
	API_requestNetTime()
	local item = g_mapItemContorl:showOnlineNode2()
end 


--getTileMapObjAtt
function g_getTileMapObjAtt(dict,map,anchorPoint)
	local key = "x"
	local x = dict[key] --(tolua.cast(dict:objectForKey(key), "cc.String")):intValue()--dynamic_cast<NSNumber*>(dict:objectForKey("x")):getNumber()
	key = "y"
	local y = dict[key]  --(tolua.cast(dict:objectForKey(key), "cc.String")):intValue()--dynamic_cast<NSNumber*>(dict:objectForKey("y")):getNumber()
	key = "width"
	local width = dict[key] --(tolua.cast(dict:objectForKey(key), "cc.String")):intValue()--dynamic_cast<NSNumber*>(dict:objectForKey("width")):getNumber()
	key = "height"
	local height = dict[key]  --(tolua.cast(dict:objectForKey(key), "cc.String")):intValue()--dynamic_cast<NSNumber*>(dict:objectForKey("height")):getNumber()
	x,y,width,height = tonumber(x),tonumber(y),tonumber(width),tonumber(height)
	local id = dict["id"] 
	local mapSize = map:getMapSize()
	local TileSize = map:getTileSize()
	--local mapY = (mapSize.height-1)*TileSize.height - y - height/2
	--cclog(mapSize.height,TileSize.height,y,height/2,"==========g_getTileMapObjAtt=================",mapY)

	if not anchorPoint then
		x =  x + width/2
		y =  y + height/2
	end 
	return x,y,width,height,id
end

--getTileMapObjAtt
function g_getTileMapObjXyByDire(atPos,nSize,map,atPos)
	

	return 
end

function g_tiledMapPosToPos(map,mapPos)
	local pos = g_tiledMapFunc:PositionFromtileCoord(map,mapPos)	
	
	return pos
end

function g_tiledPosToMapPos(map,fixPos)
	local pos = g_tiledMapFunc:tileCoordFromPosition(map,fixPos)	

	return pos
end

--计算距离
function g_MathPosXY(atPos,toPos)
	
	local absX =  toPos.x - atPos.x
	local absY =  toPos.y - atPos.y

	local absXY = absX*absX + absY*absY
	absXY = math.sqrt(absXY)
	return absXY
end 


-- ArmatureFxText 方向
function g_getNmArmatureFxText(atPos,toPos,offX)
	--local nfxType = {"d","dr","r","ur","u",}

	local nameText = "D"
	local bScaleX = 1
	if not toPos then 
		return nameText,bScaleX
	end
	local comPosX =  toPos.x - atPos.x
	local comPosY =  toPos.y - atPos.y
	
	local absXY = comPosX*comPosX + comPosY*comPosY
	absXY = math.sqrt(absXY)
	
	
	if absXY == 0 then
		return nameText,bScaleX
	end 
	
	
	local absX = math.abs(comPosX)
	local absY = math.abs(comPosY)
	local asinx = math.asin(absX/absXY)
	local asiny = math.asin(absY/absXY)
	local tanX = absX
	if absY ~= 0  then
		tanX =  absX/absY
	end 
	
	
	if  offX and absX <  offX then
		tanX = 0
	end
	local asinX  = math.deg(asinx)
	local asinY  = math.deg(asiny)
	
	local offPosX, offPosY = 0,0
	if asinX <= 15  then
		if comPosY < 0  then
			nameText = "U"
			offPosY = -1
		elseif comPosY > 0  then
			nameText = "D"
			offPosY = 1
		end 
	elseif asinX > 15 and asinX < 75 then
		if comPosY < 0  then
			nameText = "LU"
				offPosY = -1
		elseif comPosY > 0  then
			nameText = "DL"
				offPosY = 1
		end 
	else
		nameText = "L"
		offPosY = 0
	end
	
	if  offX and absY <  offX then
		offPosY = 0
	end
	
	if comPosX > 0 then
		if nameText ~= "U" and nameText ~= "D" then	
			bScaleX = 0-1
		end 
	end 
	--cclog(comPosX,"====bScaleX===11==",nameText,bScaleX,offPosX, offPosY)
	return nameText,bScaleX,offPosX, offPosY,tanX
end


--碰撞检测
function g_intersectsRect(sprite1,sprite2)
	local Scale1 = {sprite1:getScaleX(),sprite1:getScaleY()}
	local Scale2 = {sprite2:getScaleX(),sprite2:getScaleY()}
	local Parent1 =  sprite1:getParent() 
	local Parent2 =  sprite2:getParent() 
	local b_inter = false
	local curPosX1,curPosY1 = sprite1:getPosition()
	local curPosX2,curPosY2 = sprite2:getPosition()
	local pos1 = Parent1:convertToWorldSpace(cc.p(curPosX1,curPosY1))
	local pos2 = Parent2:convertToWorldSpace(cc.p(curPosX2,curPosY2))
	local size1 = sprite1:getContentSize()
	local size2 = sprite2:getContentSize()
	
	local width1 = size1.width*Scale1[1]
	local width2 = size2.width*Scale2[1]
	local height1 = size1.height*Scale1[2] 
	local height2 = size2.height*Scale2[2]

	local tbPoints = tbPoints 
	--local anchorPoint = sprite2:getAnchorPoint()
	local posRect = cc.rect( pos2.x,pos2.y,
	                                 width2+80,height2+80)
	--cclog(width1,height1,"======2222222222==========",width2,height2)
	local npos1 = cc.p(pos1.x,pos1.y)
	local npos2 = cc.p(pos1.x+width1,pos1.y)
	local npos3 = cc.p(pos1.x,pos1.y+height1)
	local npos4 = cc.p(pos1.x+width1,pos1.y+height1)
	local npos5 = cc.p(pos1.x+width1/2,pos1.y)
	local npos6 = cc.p(pos1.x+width1/2,pos1.y+height1)
	local npos7 = cc.p(pos1.x+width1,pos1.y+height1/2)
	local npos8 = cc.p(pos1.x,pos1.y+height1/2)
	local npos9 = cc.p(pos1.x+width1/2,pos1.y+height1/2)
	tbPoints = {npos1,npos2,npos3,npos4,npos5,npos6,npos7,npos8,npos9}
	for i,Point in ipairs(tbPoints)do
		
		if cc.rectContainsPoint(posRect,Point) then
			
			return true
		end	
	end

	return
end


--碰撞检测2   --简单版  圆形视野
function g_intersectsRect2(sprite1,sprite2,off)

	local curPosX1,curPosY1 = sprite1:getPosition()
	local curPosX2,curPosY2 = sprite2:getPosition()
	local pos1 = cc.p(curPosX1,curPosY1)
	local pos2 = cc.p(curPosX2,curPosY2)
	local  absXY = g_MathPosXY(pos1,pos2)
	--cclog(absXY,"==碰撞检测===",off)
	if absXY <= off then
		return  true  ,pos1,pos2, absXY
    end 	
	

	return nil ,pos1,pos2, absXY
end

function g_intersectsPoint(sprite1,Point,Scale1)
	local Scale1 = Scale1 or {1,1}

	local b_inter = false
	local pos1 = sprite1:getPosition()
	local size1 = sprite1:getContentSize()
	local width1 = size1.width*Scale1[1]
	local height1 = size1.height*Scale1[2]
	local posRect1 = cc.rect( pos1.x,pos1.y,
                                  width1,height1)					  
	if cc.rectContainsPoint(posRect1,Point) then
		return true
	end							  
	return
end


--判断层级
function g_contrastLocalZOrder(sprite1,sprite2)
	if not sprite1 or not sprite2 then
		return nil
	end 
	local Parent1 =  sprite1:getParent() 
	local Parent2 =  sprite2:getParent() 
	local b_inter = false
	local curPosX1,curPosY1 = sprite1:getPosition()
	local curPosX2,curPosY2 = sprite2:getPosition()
	local pos1 = Parent1:convertToWorldSpace(cc.p(curPosX1,curPosY1))
	local pos2 = Parent2:convertToWorldSpace(cc.p(curPosX2,curPosY2))
	if  pos2.y > pos1.y then
		return 100,101
	else
		return 101,100
	end
end

--几个text 排列
function g_tbTextConnect(tbText,MaxWidth,startPosx,offx,offy,startPosY)
	local startPosx = startPosx or 0
	local startPosY = startPosY or 0
	
	local firstText = tbText[1]
	local parent = firstText:getParent()

	--g_setAnchorPoint(parent,0,1)
	local panleWidth,panleHeight = startPosx,0
	local tbAddText = {}
	local oneTextHeight = 0
	local maxPanleWidth = 0
	for i,UiText in ipairs(tbText)do
		local textSize = UiText:getContentSize()
		UiText:setAnchorPoint(0,1)
		if i== 1 then
			oneTextHeight = textSize.height
		end
		local nextWidth = panleWidth + textSize.width
		if nextWidth <= MaxWidth then
			local posX = panleWidth
			local posY = panleHeight + startPosY
			UiText:setPosition(posX,posY)
			panleWidth = nextWidth + offx
			if maxPanleWidth  < panleWidth then
				maxPanleWidth = panleWidth
			end
		else
			local conWidth = MaxWidth - panleWidth
			maxPanleWidth = MaxWidth
			local text = UiText:getString()
			local textSwap = UiText:clone()

			local textClone2 = UiText:clone()
			local index = 1  
			local newStr = ""
			local newStr2 = ""
			local subStr = ""
			local strIndex = 1
			local numIndex = 1
			local is_end = true
			local index_max =  string.len(text)
			textClone2:setString(text)
			local textSize = textClone2:getContentSize()
			local bChange 
			if conWidth >= textSize.width - 30 then
				bChange = 0
			elseif conWidth <= 30  then
				bChange = 1
			end
			--cclog(bChange,conWidth,text,"==========bChange========",textSize.width)
			while (is_end) do  
				local temp = string.byte(text,index) 
				if (temp >= 0 and temp <= 127) then 
					subStr =string.sub(text,index,index) 
					index = index + 1 
				else 		
					subStr =string.sub(text,index,index + 2) 
					index = index + 3
				end  
				
				if not bChange  then
					local nextTextSwap = newStr..subStr
					textSwap:setString(nextTextSwap)
					local textSize = textSwap:getContentSize()
					--cclog(subStr,"=========11111================",text,textSize.width,conWidth,MaxWidth,panleWidth)
				
					if textSize.width <= conWidth  then
						newStr = newStr..subStr
					else
						newStr2 = newStr2..subStr
						bChange = 1
					end 	
				elseif bChange  == 0 then
					newStr = newStr..subStr
				else
					newStr2 = newStr2..subStr
				end
				
				
				if (index>index_max) then 
					textSwap:removeFromParent(true)
					
					UiText:setString(newStr)
					textClone2:setString(newStr2)
					is_end=false  
				end
			end 	
			local posX = panleWidth
			local posY = panleHeight + startPosY
			UiText:setPosition(posX,posY)
			panleHeight = panleHeight - textSize.height - offy
			panleWidth = startPosx
			--cclog(bChange,"==11===bChange======",posX,posY)
			if not bChange or bChange == 0 then
				
			else
				--cclog(bChange,"==2222==bChange======",panleWidth,panleHeight+ startPosY)
				textClone2:setPosition(panleWidth,panleHeight+ startPosY)
				local nTag = UiText:getTag()
				textClone2:setTag(nTag+10000)
				parent:addChild(textClone2)
				tbAddText[#tbAddText+1] = textClone2
				
				local textSize = textClone2:getContentSize()
			
				panleWidth = panleWidth + textSize.width + offx
				--cclog(newStr,newStr2,panleWidth,"======22222==============")
			end 
		end
	end
	panleHeight = 0-panleHeight
	panleHeight = panleHeight + oneTextHeight
	return panleHeight,tbAddText,maxPanleWidth
	--parent:setContentSize(cc.size(MaxWidth,panleHeight))
end


function g_getCfgID(obj)
	--cclog(type(obj),"==g_getCfgID===",obj)
	if type(obj) == "number" or type(obj) == "string" then
		return obj
	else 
		local  objID = obj:getCfgID()
		return objID
	end 
end
function g_getIconByCfgID(obj,bDrop)
	
	--cclog(icon,yanshe,lev,name,bPlist,"====g_getIconByCfgID=222====")
	return icon,yanshe,lev,name,bPlist
end
--是否简写符文
function g_getNameByCfgID(obj,runeName)
	
	return name,colorID,tbCfgmsg,icon,yanse
end

function g_getZBNameByCfgID(dbID,zbObj)
	
	return text,colorID
end 

--掉落的物品是否是物品还是资源
function g_bResOfDrop(obj)
	local status 
	local objID = g_getCfgID(obj)
	--cclog(objID,"======g_bResOfDrop==============")
	if objID >= 31100001 and objID <= 31900001 	then
		return 1
	elseif objID >= 32100001 and objID <= 32900001	then
		return 2
	elseif objID >= 33010001 and objID <= 33900006 	then
		return 3
	elseif objID >= 16000001 and objID <= 16300030 	then
		return 6	
	
	elseif objID >= 15010001 and objID <= 15540006 then
		return 5
	elseif objID >= 16400001 and objID < 16800033 then
		return 4
	elseif objID >= 10000000 and objID <= 10000003  then
		return 10	
	elseif objID >= 0 and objID <= 3  then
		return 10
	else
		return status
	end

end 
--掉落的物品是否是物品还是资源
function g_getCfgMsgByObj(obj)
	local tbCfgmsg 
	local objID = g_getCfgID(obj)
	
	return tbCfgmsg
end 
--
function g_removeFromParentToOther(item,toParent,pos)
	if not item then
		return
	end 
	local pos = pos or cc.p(0,0)
	local toParent = toParent or g_WndMgr:getrootLayout()
	item:retain()
	item:removeFromParent()
	toParent:addChild(item)
	item:setPosition(pos.x,pos.y)
	item:release()
	return item
end 
---颜色条组合
function g_tbRationLoadBar(tbLoadbar,tbNumber,countWidth)

	local count = 0
	for i,v in ipairs(tbNumber)do
		count = count + v
	end 
	g_setAnchorPoint(tbLoadbar[1],0,0)
	local nSize = tbLoadbar[1]:getContentSize()
	local height = nSize.height
	local countPosX,countPosy = tbLoadbar[1]:getPosition()
	for i,loadbar in ipairs(tbLoadbar)do
		g_setAnchorPoint(tbLoadbar[i],0,0)
		local percent = tbNumber[i]/count
		--cclog(i,percent,"==========tbNumber=======",tbNumber[i],count)
		local curWidth = countWidth*percent
		loadbar:setContentSize(cc.size(curWidth,height))
		loadbar:setPosition(countPosX,countPosy)
		countPosX = countPosX + curWidth
	end 
end

--随机x个  nType = 1 不重复 0 可重复
function g_randomTableByNumber(nTable,number,nType)
	local nType = nType or 1
	local tbMsg = {}
	for i=1,number do
		local nRandom = math.random(1,#nTable)
		
		if nType == 1 then
			tbMsg[#tbMsg + 1] = nTable[nRandom]
			table.remove(nTable,nRandom)
		else
			tbMsg[#tbMsg + 1] = nTable[nRandom]
		end
	end 
	if nType == 0  then
		local tbSortMsg = {}
		for i,v in pairs(tbMsg)do
			tbSortMsg[#tbSortMsg+1] = v
		end 
		return tbSortMsg
	end 
	return tbMsg
end


---排序，最大的前几个值
--tbale,对比的key:key = nil  即为{[1]=x,[2]=x,....}，需要前几个值，nType 1 最大，2 最小
function g_sortTableByNumber(nTable,key,number,nType)
	local nType = nType or 1
	local number = number or 1
	local swapTable = {}
	
	for i,v in pairs(nTable)do
		local value 
		if key then
			value = v[key]
			--cclog(value,v,"=========v[key]=========",v[1],v[2],v[key],key,i)
		else
			value = v
		end 
		if i == 1 then
			swapTable[1] = {i,value}
		else
			local count = 1
			local status
			while(count <= #swapTable )do
				if nType == 1 then
					if  value >= swapTable[count][2] then
						local curTb = {i,value}
						table.insert(swapTable,count,curTb)
						if #swapTable > number then
							swapTable[#swapTable] = nil
						end 
						status = true
						--cclog(swapTable,"=4444====id=======",curTb)						
						break	
					end 
				else
					if  value <= swapTable[count][2] then
						local curTb = {i,value}
						table.insert(swapTable,count,curTb)
						if #swapTable > number then
							swapTable[#swapTable] = nil
						end 
						--cclog(swapTable,"=222====id=======",curTb)
						status = true							
						break
					end 
				end
				count = count + 1
			end 
			if not status and #swapTable < number then
				local curTb = {i,value}
				swapTable[#swapTable + 1] = curTb  
				--cclog(swapTable,"=333====id=======",curTb)
			end 
		end 
	end 
	local reTable = {}
	for i=1,#swapTable do
		--cclog(swapTable[i],swapTable[i][1],swapTable[i][2],"=111====id=======",i)
		local id = swapTable[i][1]
		reTable[#reTable+1] = nTable[id]
	end 
	return reTable
end

function g_getRoleIconByCfgID(obj)
	local objID =  g_getCfgID(obj) 
	
	return icon,name,objID,anima
end 

--获取日期text
function g_getTimeDate(nTime)
	local nTime = nTime or g_getCursvTime()
	local curTab=os.date("*t",nTime); 
	local text = curTab.year.."年"..curTab.month.."月"..curTab.day.."日"
	return text
end

function g_getTimeDate2(nTime)
	local nTime = nTime or g_getCursvTime()
	local curTab=os.date("*t",nTime); 
	local text = curTab.year.."/"..curTab.month.."/"..curTab.day.."/"
	return text
end


--获取时间
function g_getTime2D(nTime,nType)
	local nType = nType or 0
	local nTime = nTime or g_getCursvTime()
	local curTab=os.date("*t",nTime); 
	local text = curTab.hour .."点"..curTab.min .."分"..curTab.sec.."秒"
	if nType == 2 then
		text = curTab.hour .."点"..curTab.min .."分"
	end 
	return text
end
--获取时间
function g_getTime2(nTime)
	local nTime = nTime or g_getCursvTime()
	local curTab=os.date("*t",nTime); 
	local text = curTab.hour ..":"..curTab.min ..":"..curTab.sec
	return text
end

--当月天数
function g_getDayAmount(nTime)
	local nTime = nTime or g_getCursvTime()
	local year,month = os.date("%Y", nTime), os.date("%m", nTime)+1
	local dayAmount = os.date("%d", os.time({year=year, month=month, day=0}))
	--cclog(dayAmount,"=====g_getDayAmount=========",year,month)
	return dayAmount
end

--获取日期
function g_getTimeDate(nTime)
	local curTab=os.date("*t",nTime); 
	local TimeStr = curTab.year.."."..curTab.month.."."..curTab.day
	return TimeStr
end
--获取日期
function g_getTimeDate2D(nTime)
	local curTab=os.date("*t",nTime); 
	local TimeStr = curTab.month.."月"..curTab.day.."日"
	return TimeStr
end

--判断日期是否是同一天
function g_bTimeDate(nTime)
	local curTime = g_getCursvTime()
	local curTab=os.date("*t",curTime); 
	local tab=os.date("*t",nTime); 
	--cclog(tab,curTab,"=====g_bTimeDate=======",nTime,curTime)
	if tab.year == curTab.year and tab.month == curTab.month and tab.day == curTab.day then
		return true
	else
		return
	end 
end 

--获取当天中午xxx点的时间
function g_getDaytime( nTime ,addTime)  
	local curTab=os.date("*t",nTime); 
	local  resultTime = os.time({year=curTab.year, month=curTab.month, day=curTab.day, hour=0,min=0,sec=0})  
	local addTime = addTime or 0
	resultTime = resultTime + addTime
    return resultTime
end 

--字符串转时间值  -- 例子： string2time(20160202)
function g_string2time( timeString )  
    local Y = string.sub(timeString , 1, 4)  
    local M = string.sub(timeString , 5, 6)  
    local D = string.sub(timeString , 7, 8)  
    return os.time({year=Y, month=M, day=D, hour=0,min=0,sec=0})  
end 

--判断日期是否是在时间段内  --参数：(20160202 ,60*60*24)
function g_bTimeDateInTime(startTimeStr,endTime)
	local curTime = g_getCursvTime()
	local nTime = g_string2time(startTimeStr)  
	local endTime1 = nTime + endTime
	
	--cclog(endTime1,nTime,curTime,"=====g_bTimeDateInTime========",startTimeStr,endTime)
	if curTime >= nTime and curTime <= endTime1 then
		return true
	end 
	return
end 

function setUserHeadIcon(imageview, url, openid)
	local realURL =string.format("%s?%d=%d.jpg", url, openid, openid)
	--cclog(imageview,openid,"=======setUserHeadIcon=========")
	if imageview and openid then
		API_DownLoadImg(imageview, realURL, tostring(openid))
	end
end

function g_suploadlog0(lName)
	--ReportBuglyLog()	
	
	local pathToSave = cc.FileUtils:getInstance():getWritablePath()
	local gameID =  ""
	local userid =  cc.UserDefault:getInstance():getIntegerForKey("userid", 0)
	if userid == 0 then
		userid = API_GetDeviceID()
	end 
	local toFilePath
	local logName = cc.UserDefault:getInstance():getStringForKey("lastLogFile", "")
	local filePath = pathToSave.."LogFiles/"..logName
	local curVersion =  cc.UserDefault:getInstance():getStringForKey("current-version-code",  get_szAppVersion())
	local LogFileName = cc.UserDefault:getInstance():getIntegerForKey("LogFileName", 1)
	logName = lName
	 cclog(g_astSupLog,"=====LogFileName=====",logName)
	if not logName or logName ~= g_astSupLog then
		LogFileName = LogFileName + 1
		g_astSupLog  = logName
		cc.UserDefault:getInstance():setIntegerForKey("LogFileName", LogFileName)
		cc.UserDefault:getInstance():setStringForKey("suploadLogFileName", logName)
	end 
	local logName = userid.."_"..LogFileName.."_"..curVersion
	toFilePath = filePath
	local svrIp = publicSvrIp -- 
	local  rootSvrIp = string.format("http://%s/", svrIp)
	local  rootSvrIp =  g_GameDataMgr.rootSvrIp 
		
	network.uploadFile(
		function(evt)
			if evt.name == "completed" then
				local request = evt.request
				release_print("REQUEST getResponseStatusCode() = %d", request:getResponseStatusCode())
				--release_print("REQUEST getResponseHeadersString() =\n%s", request:getResponseHeadersString())
				--release_print("REQUEST getResponseDataLength() = %d", request:getResponseDataLength())
				--release_print("REQUEST getResponseString() =\n%s", request:getResponseString())			
			elseif evt.name == "progress" then
				--print(evt.total.." "..evt.dltotal)
				--evt.request:delete()
			end
		end,
		
	
		rootSvrIp.."html/uploadfile",	
		{
			fileFieldName="filepath",
			filePath = toFilePath,
			contentType = "multipart/form-data",
			fileBuffName = logName..".log",
			extra={
				{"op", "upload"},
				{"act", "upload"},
			}
		}
	)

end


--获得资源飘字
function g_ziyuanPiaozi(ntype,valueText,scale,pos,parent)
	local Panel_Text_ = g_WndMgr:getWnd(1).rootLayout:getChildByName("Panel_Text_")
	local Text_clone = Panel_Text_:clone()
	local parent = parent or g_WndMgr:getrootLayout()
	parent:addChild(Text_clone)
	Text_clone:setLocalZOrder(G_maxLocalZOrder)
	
	local pos = pos or cc.p(300,680)
	Text_clone:setPosition(pos.x,pos.y)
	g_action:ts_hszz_antion2(Text_clone)
	
	local iconPath,bPlistType =  g_iconContorl:getZhiyuanIcon(ntype)
	local Image_331 = Text_clone:getChildByName("Image_331")
	Image_331:loadTexture(iconPath,bPlistType)
	if scale then
		Text_clone:setScale(scale)
	end 
	local Text_qinwa = Text_clone:getChildByName("Text_qinwa")
	Text_qinwa:setString("+"..valueText)
	return Text_qinwa
end

--获取nType
function g_getZBType(cfgID)
	local nType = math.floor(cfgID/1000000)
	nType = nType %10
	if nType == 2 then
		return 3
	elseif nType == 3 then
		return 2
	elseif nType == 1 then
		return 1
	end 

	return 
end

function nRandomseed()  
	math.randomseed(tostring(os.time()):reverse():sub(1, 6))
end 

--获得物品飘字  tbDropCfgId[1] = {id = tbDropCfgId , value1 = number ,value2 = addnumber} 
function g_getGoodsPiaozi(tbDropCfgId,count,bset)
	local count = count or -1
	for i,v in pairs(tbDropCfgId)do	
		local goodsCfgId = v.id
		local number = v.value1
		local addNumber = v.value2
		local nType = math.floor(goodsCfgId/1000000)
		--cclog(goodsCfgId,nType,"=======goodsCfgId==========")
		if nType == 15 then --fuwem
			local fwObj = g_fuwenObjData:createfuwenObj(goodsCfgId,addNumber) 
			local name,colorID = g_getNameByCfgID(goodsCfgId)
			if addNumber > 1 then
				g_action:ts_hszz_antion("获得:"..name.."*"..addNumber,colorID,count)
			else
				g_action:ts_hszz_antion("获得:"..name,colorID,count)
			end 
			fwStatus = true
		elseif nType == 11 then --goods
			local name,colorID = g_getNameByCfgID(goodsCfgId)
			
			if addNumber > 1 then
				g_action:ts_hszz_antion("获得:"..name.."*"..addNumber,colorID,count)
			else
				g_action:ts_hszz_antion("获得:"..name,colorID,count)
			end 
			if goodsCfgId >= 16000001 and goodsCfgId <= 16300030 then
				g_plantObjData:addTbplantObjDataOfNumsByCfgId(goodsCfgId,number,nil,true) 
			else
				g_bagDaoJuObjData:addTbbagDaoJuObjDataOfNumsByCfgId(goodsCfgId,number,nil,true)
			end 
		elseif nType == 31 or  nType == 32 or  nType == 33 then
			
			
		elseif goodsCfgId <= 4 then --res
			local name = g_tbGoods[goodsCfgId]
			g_action:ts_hszz_antion("获得:"..name.."+"..addNumber,nil,count)
			g_gameContorl:setRes(goodsCfgId,number)
		end 
		count = count + 1
	end 
		
	return count
end 




local screenfileName = "printScreen.png"
function g_GetScreenFileName()	
	local writablePath = cc.FileUtils:getInstance():getWritablePath()
	return writablePath..screenfileName
end

function g_Screen(node, callBack)
	local fileName = g_GetScreenFileName()
	-- 截屏
	cc.utils:captureScreen(function(succeed, outputFile)
		if node then
			node:removeFromParent()
		end
		
		if succeed then
			print("截屏成功"..outputFile)
			
		else
			print("截屏失败")
		end
		
		if callBack then
			callBack(succeed, outputFile)
		end
	end, fileName)
end
---- 
--https://blog.csdn.net/zwc2xm/article/details/50675387 


function RenderTexture_onSaveToFile(strFileName)
	cclog("RenderTexture_onSaveToFile"..strFileName)
	
end


function g_Screen2(windows, addNode, callback)
	local filename = screenfileName	
	windows:retain()
	local size = windows:getContentSize() -- 返回size
	 --创建渲染
	renderTexture = cc.RenderTexture:create(size.width, size.height, cc.TEXTURE2_D_PIXEL_FORMAT_RGB_A8888)
	renderTexture:begin()

	windows:visit()
	addNode:visit()
	renderTexture:endToLua()--这个之后必须隔帧取(或者加个定时器 当前帧取到的数据全是0)
	renderTexture:retain()--必须加这个 保存对象 否则下帧就被回收了 不需要了记得release
	local bret = renderTexture:saveToFile(filename, cc.IMAGE_FORMAT_PNG, true, function(picpath)
			--cclog("g_Screen2 "..picpath)
			addNode:release()
			windows:release()
			renderTexture:release()
			if callBack then
				callBack(true, picpath)
			end
		end
	)
	--cclog(filename, tostring(bret))
end

--[[比较两个时间，返回相差多少时间]]  
function timediff(long_time,short_time)  
	local n_short_time,n_long_time,carry,diff = os.date('*t',short_time),os.date('*t',long_time),false,{}  
	local colMax = {60,60,24,os.date('*t',os.time{year=n_short_time.year,month=n_short_time.month+1,day=0}).day,12,0}  
	n_long_time.hour = n_long_time.hour - (n_long_time.isdst and 1 or 0) + (n_short_time.isdst and 1 or 0) -- handle dst  
	for i,v in ipairs({'sec','min','hour','day','month','year'}) do  
		diff[v] = n_long_time[v] - n_short_time[v] + (carry and -1 or 0)  
		carry = diff[v] < 0  
		if carry then  
			diff[v] = diff[v] + colMax[i]
		end  
	end  
	return diff  
end  

local function dumpWidget(root, space)
	local name = root:getName()
	local tbWidgets = {}
	if root.getChildren then
		tbWidgets = root:getChildren()
	end 
		
	local tips = name.."\n{\n"
	for key, value in pairs(tbWidgets) do
		tips = tips..space..key.." "..value:getName()..'\n'
		--print(key.." "..value:getName())
		dumpWidget(value, space..'    \n')
	end
	
	tips = tips..'}'
	return tips
end




function dumpFairGuiWidget(root)
	print(dumpWidget(root, "    "))
end


local tbZzGameValue = {}
function zzGameValue_(zzGameID)
	if not tbZzGameValue[zzGameID] then
		local cfgMsg =  g_cfgMsg:getCfgMsg("zz_game_D",zzGameID)
		local tbValue  = cfgMsg.value
		local tbStrNumber = cc.g_inalienContentByCfgMsg(tbValue,1,"_")
		tbZzGameValue[zzGameID] = tbStrNumber
	end 
	return tbZzGameValue[zzGameID]
end

function getChildById(root, id,isLog)
	if not root then
		cclog("=getChildById=not root=")
		return nil
	end
	local chidren = root:getChildren()
	for i =1, #chidren do
		local curChild = chidren[i] 
		if isLog then
			cclog(curChild:getCustomData(),"=====getChildById=====",id)
		end 
		if curChild:getCustomData() == id then
			return curChild
		end
	end
	
	return nil
end


---权重通用  -- tbRation = ration   
function getRandomBytbRation(tbRation,maxValue)
	local nRandom = math.random(1,maxValue)
	local count = 0
	for i, v  in  ipairs(tbRation)do
		count = count + v 
		if   nRandom <=  count  then
			return i
		end 
	end 
	return   
end

---权重通用  -- tbRation = {  {key, ration} }   
function getRandomBytbRation2(tbRation,maxValue)
	local nRandom = math.random(1,maxValue)
	local count = 0
	for i, v  in  pairs(tbRation)do
		count = count + v[2]
		if   nRandom <=  count  then
			return v[1],i
		end 
	end 
	return   
end


--ngx.say(getWeekNum("2018-04-25"))
--根据日期获取星期几
function getWeekNum(strDate)
    local ymd = Split(strDate,"-")
    local t = os.time({year=tostring(ymd[1]),month=tostring(ymd[2]),day=tostring(ymd[3])})
    local weekNum = os.date("*t",t).wday  -1
    if weekNum == 0 then
        weekNum = 7
    end
    return weekNum
end
--根据日期获取星期几
function getWeekNum2(nTime)
    local t = nTime or g_getCursvTime()
    local weekNum = os.date("*t",t).wday  -1
    if weekNum == 0 then
        weekNum = 7
    end
    return weekNum
end

function handler(target, method)
    return function(...) return method(target, ...) end
end


--分割字符串
function g_split( str,reps )
    local resultStrList = {}
    string.gsub(str,'[^'..reps..']+',function ( w )
        table.insert(resultStrList,w)
    end)
    return resultStrList
end

function string.split(input, delimiter)
    input = tostring(input)
    delimiter = tostring(delimiter)
    if (delimiter=='') then return false end
    local pos,arr = 0, {}
    for st,sp in function() return string.find(input, delimiter, pos, true) end do
        table.insert(arr, string.sub(input, pos, st - 1))
        pos = sp + 1
    end
    table.insert(arr, string.sub(input, pos))
    return arr
end


function isEmpty(t)
	return next(t) == nil
end

--判断是空表
function table_is_empty(t)
	return next(t) == nil
end

--判断是否有效字符串
function g_isValidStr(str)

	if str and string.len(str) > 0 then
		return true
	end

	return false
end



--table长度
function g_getTbLen(tbMsg)

	if tbMsg then
		local count = 0
		for i,v in pairs(tbMsg)do
			count = count + 1
		end 
		return  count
	end

	return 
end

function getTbCount(nTable)
	local  count = 0
	local nTable = nTable or {}
	for i,v in pairs(nTable)do 
		count = count + 1
	end
	return  count
end


function release(resource)
	if type(resource) == "table" then
		  for k,v in pairs(resource) do
			   if type(v) == "table" then
					  release(v)
			   else
					 resource[k] = nil
			   end
		  end
	else
		 resource = nil     
	end
end
local isCountTest = nil
--输出内存大小

function cclogColle(Tag,Tag2,Tag3)
	if not isCountTest then
		return 
	end 
	local  casColle1 = collectgarbage("count")
	collectgarbage("collect")
	local  casColle2 = collectgarbage("count")
	local casColle = casColle2 - casColle1
	---cclog(casColle,"===cclogColle========",Tag,Tag2,Tag3,casColle2,casColle1)
	return casColle2
end

local iLoginClickTime = nil
function CheckLoginClick()
    cclog("=CheckLoginClick====",iLoginClickTime)
	if not iLoginClickTime then
		iLoginClickTime = os.time()
		return 
	else
		local iCurTime  = os.time()
		local iLeft = 1 - (iCurTime - iLoginClickTime)
            
		if iLeft > 0 then
			g_action:ts_hszz_antion(string.format("点击太快了 %d秒后再试", iLeft), nil,-2,nil,nil,rootTips)
			return true
		end
		
		iLoginClickTime = iCurTime
		return 
	end
end



g_function = true